//This version modified for Poisson
DATA_SECTION
//Read in data from "albatross.dat"
// albatross data contains number of trawls with 0, 1,..., maxC albatrosses caught
  init_int maxC  //maximum number caught
  init_matrix Catchdat(0,maxC,1,2)
  number Tottows

//Define looping variables in the data section, so you only have to define them once.
//Because they are in the data section they are valid everywhere in the model.
  int i; 

// These lines were used to check the data came in ok
// !!cout<<Catchdat;
// !!exit(5);

PARAMETER_SECTION
// lam is poisson parameter and is both mean and variance of distribution
  init_number lnlam
  number lam

  vector ProbCatch(0,maxC) // to contain probability of each catch
  vector PredCount(0,maxC) // to contain predicted number of trawls with catch

//Value for the objective function to minimize
  objective_function_value negLogL    //concentrated likelihood value

INITIALIZATION_SECTION
//Starting values for parameters
  lnlam -1.3
 

PRELIMINARY_CALCS_SECTION
  for(i=0;i<=maxC;i++)
  {
  Tottows+=Catchdat(i,2);
  }

PROCEDURE_SECTION
  lam=exp(lnlam);

  for(i=0;i<=maxC;i++)
  {
     negLogL+=Catchdat(i,2)*(lam-Catchdat(i,1)*lnlam+log(factorial(Catchdat(i,1))));
     ProbCatch(i)=exp(-lam+Catchdat(i,1)*lnlam-log(factorial(Catchdat(i,1))));  
  }
  PredCount=Tottows*ProbCatch;

// THERE IS NO FACTORIAL FUNCTION IN ADMB SO WILBERG BUILT THIS ONE
FUNCTION double factorial(double n)
  double fact;
  int k; 
  fact=1.0;
  if(n!=0.0)
  {
    for (k=1;k<=n;k++)
    {
      fact*=k;
    }
  }
  return(fact);

REPORT_SECTION
  report << "lam = Poisson mean and variance" << endl;
  report << lam << endl;
  report << "Prob of catch for catch of 0, 1, ..., maxC" << endl;
  report << ProbCatch << endl;
  report << "Predicted number of trawls for catch of 0, 1, ..., maxc" << endl;
  report << PredCount << endl;
 
