//Albatross by-catch model from Hilborn and Mangel 1997
//Developed for ADMB Short Course 1, June 2006
//Created 6-3-06 by J. Bence
//Last modified 6-5-06 by J. Bence and B. Linton

DATA_SECTION
//Read in data from "albatross.dat"
// albatross data contains number of trawls with 0, 1,..., maxC albatrosses caught
  init_int maxC  //maximum number caught
  init_matrix Catchdat(0,maxC,1,2)

//Quantity calculated from input data
  number Tottows //total number of trawl tows

//Define looping variables in the data section, so you only have to define them once.
//Because they are in the data section they are valid everywhere in the model.
  int i; 

// These lines were used to check the data came in ok
// !!cout<<Catchdat;
// !!exit(5);

PARAMETER_SECTION
// m and s are negative binomial mean and overdispersion parameters
  init_number lnm  //log scale m
  init_number lns  //log scale s
  number m
  number s
  vector ProbCatch(0,maxC)  //to contain probability of each catch
  sdreport_vector PredCount(0,maxC)  //to contain predicted number of trawls with catch

//Value for the objective function to minimize
  objective_function_value negLogL    //concentrated likelihood value

INITIALIZATION_SECTION
//Starting values for parameters
  lnm -1.3
  lns -4.5

PRELIMINARY_CALCS_SECTION
//Calculations that only need to be made once
  //Loop calculates total number of albatross caught
//  for(i=0;i<=maxC;i++)
//  {
//  Tottows+=Catchdat(i,2);
//  }
// This avoids a loop
  Tottows=sum(extract_column(Catchdat,2));

PROCEDURE_SECTION
  m=exp(lnm);
  s=exp(lns);
  for(i=0;i<=maxC;i++)
  {
     //Objective function value (negative binomial) to be minimized
     negLogL+=Catchdat(i,2)*(-gammln(Catchdat(i,1)+s)+gammln(s)-s*log(s/(m+s))-Catchdat(i,1)*log(m/(m+s)));
     // Calculate probability of each catch
  // IF SD_PHASE MEANS THESE ARE ONLY CALCULATED DURING PHASE WHEN ASYMP SDS ARE CALCULATED
    if (sd_phase()) {
      ProbCatch(i)=exp(gammln(Catchdat(i,1)+s)-gammln(s)+s*log(s/(m+s))+Catchdat(i,1)*log(m/(m+s))-log(factorial(Catchdat(i,1))));
    }
   }
  if (sd_phase()) {PredCount=Tottows*ProbCatch;}  //calculate predicted number of trawls with catch

// THERE IS NO FACTORIAL FUNCTION IN ADMB SO WILBERG BUILT THIS ONE
FUNCTION double factorial(double n)
  double fact;  //declare local variable (c++ double=ADMB number)
  int k;  //declare local variable
  fact=1.0;
  if(n!=0.0)  //if n not equal to zero
  {
    for (k=1;k<=n;k++)
    {
      fact*=k;
    }
  }
  return(fact);


REPORT_SECTION
//Write data to "albatross.rep" file

// THESE NEED TO BE CALCULATED HERE SINCE REP FILE IS WRITTEN BEFORE SD PHASE
  for(i=0;i<=maxC;i++)
  {
   // Calculate probability of each catch
   ProbCatch(i)=exp(gammln(Catchdat(i,1)+s)-gammln(s)+s*log(s/(m+s))+Catchdat(i,1)*log(m/(m+s))-log(factorial(Catchdat(i,1))));
  }
  PredCount=Tottows*ProbCatch;  //calculate predicted number of trawls with catch
  report << "Neg Binomial Mean" << endl;
  report << m << endl;
  report << "Neg Binomial Overdispersion Parameter" << endl;
  report << s << endl;
  report << "Prob of catch for catch of 0, 1, ..., maxC" << endl;
  report << ProbCatch << endl;
  report << "Predicted number of trawls for catch of 0, 1, ..., maxc" << endl;
  report << PredCount << endl;
 
