// SURPLUS PRODUCTION EXAMPLE USING ARROWTOOTH FLOUNDER MODEL
// MODIFIED PREPARED FOR JUNE 2006 SHORT COURSE BY JIM BENCE
// MODIFIED SEVERAL TIMES FROM CODE BY SCOTT BOOMER USED IN 2000
// CORNELL WORKSHOP 
// THIS VERSION FOR MEASUREMENT ERROR AND LOGNORMAL ERROR
// REQUIRED SOME PENALTIES -- NOT IN PLAY AT SOLUTION

DATA_SECTION
  init_number nobs
  init_vector Bindex(0,nobs-1)
  init_vector Yobs(0,nobs-1)
// GLOBALLY DEFINE INCREMENTER FOR COMPLILER TRANSPORTABILITY
  int i 

PARAMETER_SECTION
// Good practice to bound all params if you envision any MCMC...
// Params estimated on log scale
  init_bounded_number lnBinf(2,10) //ln CARRYING CAPACITY FOR BIOMASS
  init_bounded_number lnm(2,10)    // ln MAX PRODUCTIVITY
  init_bounded_number lnB0(2,10)   // ln BIOMASS AT START OF FIRST YEAR
  init_number lnq(-1)              // ln of catchability

  number Binf  
  number m    
  number q    
  number Blast // Biomass at start of year after data end

  vector B(0,nobs-1);          // Biomass at start of each year
  vector pred_index(0,nobs-1); // This equals B if q=1
  vector Y(0,nobs-1);          // Set to max of obs Y or 50% of biomass
  vector dP(0,nobs-1);         // PRODUCTIVITY
  number RSS;             // RESIDUAL SUM OF SQUARES

  objective_function_value negconc;  // this is what gets minimized

INITIALIZATION_SECTION
// SET INITIAL VALUES OF PARAMS HERE
  lnBinf 6.2
  lnm     4.0
  lnB0   4.5
  lnq  0

PROCEDURE_SECTION
  get_B();
  get_obj();

FUNCTION get_B
// THIS FUNCTION CALCULATES BIOMASS TRAJECTORY GIVEN PARAMETER VALUES
// FIRST WE BACK TRANSFORM PARAMETERS OUT OF LOG SPACE
 Binf=exp(lnBinf);
 m=exp(lnm);
 q=exp(lnq);
 B(0)=exp(lnB0);
  for (i=0;i< nobs-1;i++)	
  {
  // dP IS PRODUCTION, I.E., CHANGE IF NO FISHING
    dP(i) = (4*m/Binf)*(1-B(i)/Binf)*B(i);
 // CODE ASSUMES YIELD > HALF STOCK UNREASONABLE
	if (Yobs(i)>0.5*B(i)) Y(i)=0.5*B(i);
	else Y(i)=Yobs(i);
 // JUST IN CASE NEG BIOMASS
//	if (Y(i)<1.0) Y(i)=1.0;
    B(i+1) = B(i)+ dP(i)-Y(i);
 // AND BIOMASS CAN'T BE TOO NEAR ZERO;
	if (B(i+1)<10.0) B(i+1)=10.0;
  } //end for

// MULTIPLIES B BY q TO GET PREDICTED VALUES OF INDEX;
  pred_index=q*B;

// THIS FUNCTION CALULATES OBJECTIVE FUNC (NEG LOG CONC LIKELIHOOD)
FUNCTION get_obj
// norm2 sums the squared elements of a vector
  RSS=norm2(log(pred_index)-log(Bindex));
 
 // PENALTY FOR ADJUSTMENTS TO OBSERVED YIELD
 // INTENDED TO PREVENT GETTING STUCK WHERE YOU ARE RUNNING OUT OF FISH
 // NEED TO CHECK THAT THIS PENALTY IS NOT DOING ANYTHING AT SOLUTION
  RSS+=norm2(log(Y(0,nobs-2))-log(Yobs(0,nobs-2)));  

  negconc=(nobs/2.0)*log(RSS);
 
REPORT_SECTION
// NOT NEEDED TO FIT MODEL BUT WE CAN PROJECT ONE MORE YEAR
  Y(nobs-1)=Yobs(nobs-1);
  Blast=B(nobs-1)+(4*m/Binf)*(1-B(nobs-1)/Binf)*B(nobs-1)-Y(nobs-1);
  report << "Binf" << endl;
  report <<  Binf  << endl;
  report << "m" << endl;
  report <<  m  << endl;
  report << "biomass" << endl;
  report << B << endl;
  report << "projected biomass after modeled period" << endl;
  report << Blast << endl;
  report << "observed survey biomass (index)" << endl;
  report << Bindex << endl;
  report << "pred yield" << endl;
  report << Y << endl;
  report << "ratio of start to end biomass" << endl;
  report << B(0)/Blast;

