//von Bertalanffy growth model
//Developed for ADMB Short Course 1, June 2006
//Created 5-12-06 by B. Linton
//Last modified 6-21-06 by B. Linton

//Any text after "//" is ignored

DATA_SECTION
//Read in data from "growth.dat" file
  init_int nobs  //number of observations in data set
  init_vector ages(1,nobs)  //observed ages
  init_vector length_obs(1,nobs)  //observed mean length-at-age

PARAMETER_SECTION
//Parameters to be estimated in model
  init_number log_Linf  //asymptotic length (log scale)
  init_number log_K  //growth coefficient (log scale)
  init_number t0  //age at which length is 0cm

//Quantities to be calculated from parameters
  number K  //Brody growth coefficient
  number Linf //asymptotic size
  vector length_pred(1,nobs)  //predicted mean length-at-age
  number rss //residual sum of squares

// The objective function - we will minimize concentrated negative log likelihood  
  objective_function_value conc_like  //concentrated negative log likelihood

INITIALIZATION_SECTION
//Starting values for parameters
  log_Linf 3.4
  log_K -1.0
  t0 -0.1

PROCEDURE_SECTION
//Bactransform parameter
  K=exp(log_K);
  Linf=exp(log_Linf);

//von Bertalanffy growth model
  length_pred=Linf*(1.0-exp(-K*(ages-t0)));

//Objective function defined
  //Function norm2(x) = sum of x^2
  rss=norm2(log(length_obs)-log(length_pred)); //residual sum of squares
  
  //Minimize negative log concentrated likelihood
  conc_like=(nobs/2.0)*log(rss);


REPORT_SECTION
//By default every thing in this section gets written to the file "growth.rep"
//Characters in quotation marks are written out as text
  report << "Predicted mean length-at-age" << endl;
  report << length_pred << endl;
  report << "Observed mean length-at-age" << endl;
  report << length_obs << endl;
  report << "von Bertalanffy growth model parameters" << endl;
  report << "Asymptotic length (Linf)" << endl;
  report << Linf << endl;
  report << "Growth coefficient (K)" << endl;
  report << K << endl;
  report << "Age of length 0cm (t0)" << endl;
  report << t0 << endl;
  report << "Ratio of length at age-1 to length at age-10" << endl;
  report << length_pred(1)/length_pred(nobs) << endl;
